@@ -2588,14 +2588,12 @@ handle_telemetry_cmd_ipsec_secgw_stats(const char *cmd __rte_unused,
 	}
 
 	/* add telemetry key/values pairs */
-	rte_tel_data_add_dict_u64(data, "packets received",
-				total_pkts_rx);
+	rte_tel_data_add_dict_uint(data, "packets received", total_pkts_rx);
 
-	rte_tel_data_add_dict_u64(data, "packets transmitted",
-				total_pkts_tx);
+	rte_tel_data_add_dict_uint(data, "packets transmitted", total_pkts_tx);
 
-	rte_tel_data_add_dict_u64(data, "packets dropped",
-				total_pkts_dropped);
+	rte_tel_data_add_dict_uint(data, "packets dropped",
+				   total_pkts_dropped);
 
 
 	return 0;
@@ -2695,30 +2693,30 @@ handle_telemetry_cmd_ipsec_secgw_stats_outbound(const char *cmd __rte_unused,
 
 	/* add spd 4 telemetry key/values pairs */
 
-	rte_tel_data_add_dict_u64(spd4_data, "protect",
-		total_stats.outbound.spd4.protect);
-	rte_tel_data_add_dict_u64(spd4_data, "bypass",
-		total_stats.outbound.spd4.bypass);
-	rte_tel_data_add_dict_u64(spd4_data, "discard",
-		total_stats.outbound.spd4.discard);
+	rte_tel_data_add_dict_uint(spd4_data, "protect",
+				   total_stats.outbound.spd4.protect);
+	rte_tel_data_add_dict_uint(spd4_data, "bypass",
+				   total_stats.outbound.spd4.bypass);
+	rte_tel_data_add_dict_uint(spd4_data, "discard",
+				   total_stats.outbound.spd4.discard);
 
 	rte_tel_data_add_dict_container(data, "spd4", spd4_data, 0);
 
 	/* add spd 6 telemetry key/values pairs */
 
-	rte_tel_data_add_dict_u64(spd6_data, "protect",
-		total_stats.outbound.spd6.protect);
-	rte_tel_data_add_dict_u64(spd6_data, "bypass",
-		total_stats.outbound.spd6.bypass);
-	rte_tel_data_add_dict_u64(spd6_data, "discard",
-		total_stats.outbound.spd6.discard);
+	rte_tel_data_add_dict_uint(spd6_data, "protect",
+				   total_stats.outbound.spd6.protect);
+	rte_tel_data_add_dict_uint(spd6_data, "bypass",
+				   total_stats.outbound.spd6.bypass);
+	rte_tel_data_add_dict_uint(spd6_data, "discard",
+				   total_stats.outbound.spd6.discard);
 
 	rte_tel_data_add_dict_container(data, "spd6", spd6_data, 0);
 
 	/* add sad telemetry key/values pairs */
 
-	rte_tel_data_add_dict_u64(sad_data, "miss",
-		total_stats.outbound.sad.miss);
+	rte_tel_data_add_dict_uint(sad_data, "miss",
+				   total_stats.outbound.sad.miss);
 
 	rte_tel_data_add_dict_container(data, "sad", sad_data, 0);
 
@@ -2769,30 +2767,30 @@ handle_telemetry_cmd_ipsec_secgw_stats_inbound(const char *cmd __rte_unused,
 
 	/* add sad telemetry key/values pairs */
 
-	rte_tel_data_add_dict_u64(sad_data, "miss",
-		total_stats.inbound.sad.miss);
+	rte_tel_data_add_dict_uint(sad_data, "miss",
+				   total_stats.inbound.sad.miss);
 
 	rte_tel_data_add_dict_container(data, "sad", sad_data, 0);
 
 	/* add spd 4 telemetry key/values pairs */
 
-	rte_tel_data_add_dict_u64(spd4_data, "protect",
-		total_stats.inbound.spd4.protect);
-	rte_tel_data_add_dict_u64(spd4_data, "bypass",
-		total_stats.inbound.spd4.bypass);
-	rte_tel_data_add_dict_u64(spd4_data, "discard",
-		total_stats.inbound.spd4.discard);
+	rte_tel_data_add_dict_uint(spd4_data, "protect",
+				   total_stats.inbound.spd4.protect);
+	rte_tel_data_add_dict_uint(spd4_data, "bypass",
+				   total_stats.inbound.spd4.bypass);
+	rte_tel_data_add_dict_uint(spd4_data, "discard",
+				   total_stats.inbound.spd4.discard);
 
 	rte_tel_data_add_dict_container(data, "spd4", spd4_data, 0);
 
 	/* add spd 6 telemetry key/values pairs */
 
-	rte_tel_data_add_dict_u64(spd6_data, "protect",
-		total_stats.inbound.spd6.protect);
-	rte_tel_data_add_dict_u64(spd6_data, "bypass",
-		total_stats.inbound.spd6.bypass);
-	rte_tel_data_add_dict_u64(spd6_data, "discard",
-		total_stats.inbound.spd6.discard);
+	rte_tel_data_add_dict_uint(spd6_data, "protect",
+				   total_stats.inbound.spd6.protect);
+	rte_tel_data_add_dict_uint(spd6_data, "bypass",
+				   total_stats.inbound.spd6.bypass);
+	rte_tel_data_add_dict_uint(spd6_data, "discard",
+				   total_stats.inbound.spd6.discard);
 
 	rte_tel_data_add_dict_container(data, "spd6", spd6_data, 0);
 
@@ -2839,14 +2837,12 @@ handle_telemetry_cmd_ipsec_secgw_stats_routing(const char *cmd __rte_unused,
 	update_statistics(&total_stats, coreid);
 
 	/* add lpm 4 telemetry key/values pairs */
-	rte_tel_data_add_dict_u64(lpm4_data, "miss",
-		total_stats.lpm4.miss);
+	rte_tel_data_add_dict_uint(lpm4_data, "miss", total_stats.lpm4.miss);
 
 	rte_tel_data_add_dict_container(data, "IPv4 LPM", lpm4_data, 0);
 
 	/* add lpm 6 telemetry key/values pairs */
-	rte_tel_data_add_dict_u64(lpm6_data, "miss",
-		total_stats.lpm6.miss);
+	rte_tel_data_add_dict_uint(lpm6_data, "miss", total_stats.lpm6.miss);
 
 	rte_tel_data_add_dict_container(data, "IPv6 LPM", lpm6_data, 0);
 
